'''
Mapping from camera type string to a module/class
that can act as a specialized client for that DM type
'''
from plico.client.discovery import ClientMapType


client_map = {
   'Ocam2k': ClientMapType(modulename='pysilico.client.ocam2k_client', classname='Ocam2kClient'),
   'CblueOneCamera': ClientMapType(modulename='pysilico.client.cblue_client', classname='CBlueClient'),
}



#!/usr/bin/env python
import unittest
import numpy as np
from pysilico.types.camera_frame import CameraFrame


class CameraFrameTest(unittest.TestCase):

    def setUp(self):
        self._createCameraFrame()

    def _createCameraFrame(self):
        self._frame = np.arange(
            120 * 100, dtype=np.uint8).reshape((120, 100))
        self._counter = 42
        self._cameraFrame = CameraFrame.fromNumpyArray(self._frame,
                                                       self._counter)

    def testNumpy(self):
        self._createCameraFrame()
        frame2 = self._cameraFrame.toNumpyArray()
        self.assertTrue(np.array_equal(self._frame, frame2))

    def testTypeIsUint16(self):
        self.assertEqual(np.uint16, self._cameraFrame.toNumpyArray().dtype)

    def testComparison(self):
        counter = 42
        cameraFrame2 = CameraFrame.fromNumpyArray(self._frame.copy(), counter)
        self.assertTrue(self._cameraFrame == cameraFrame2)

    def testStringRepresentation(self):
        self.assertEqual('frame-42', str(self._cameraFrame))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

from pysilico.client.camera_client import CameraClient
from pysilico.utils.timeout import Timeout


class CBlueClient(CameraClient):

    def __init__(self, rpcHandler, sockets):
        super().__init__(rpcHandler, sockets)

    def set_pixel_format(self, pixel_format, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'set_pixel_format', [pixel_format],
            timeout=timeoutInSec)

    def get_pixel_format(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'get_pixel_format', [],
            timeout=timeoutInSec)
    
    def set_rows(self, rows, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'set_rows', [rows],
            timeout=timeoutInSec)
    
    def get_rows(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'rows', [],
            timeout=timeoutInSec)
    
    def get_rows_max(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'get_rows_max', [],
            timeout=timeoutInSec)

    def set_cols(self, cols, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'set_cols', [cols],
            timeout=timeoutInSec)
    
    def get_cols(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'cols', [],
            timeout=timeoutInSec)
    
    def get_cols_max(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'get_cols_max', [],
            timeout=timeoutInSec)

    def set_cooling_setpoint(self, set_point, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'set_device_cooling_setpoint', [set_point],
            timeout=timeoutInSec)
    
    def get_cooling_setpoint(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'get_device_cooling_setpoint', [],
            timeout=timeoutInSec)
    
    def set_conversion_efficiency(self, conv_eff, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'set_conversion_efficiency', [conv_eff],
            timeout=timeoutInSec)
    
    def get_conversion_efficiency(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'get_conversion_efficiency', [],
            timeout=timeoutInSec)

    def set_gain(self, gain, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'set_gain', [gain],
            timeout=timeoutInSec)
    
    def get_gain(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'get_gain', [],
            timeout=timeoutInSec)

    def set_framerate(self, framerate, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'setFrameRate', [framerate],
            timeout=timeoutInSec)
    
    def get_framerate(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'getFrameRate', [],
            timeout=timeoutInSec)
    
    def get_framerate_min(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'getFrameRateMin', [],
            timeout=timeoutInSec)
    
    def get_framerate_max(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self._requestSocket,
            'getFrameRateMax', [],
            timeout=timeoutInSec)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'camera_control_gui.ui',
# licensing of 'camera_control_gui.ui' applies.
#
# Created: Wed Sep 12 17:51:00 2018
#      by: pyside2-uic  running on PySide2 5.11.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

from PyQt5.QtWidgets import QApplication as QtCompat


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(505, 584)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.lineEditServerHostName = QtWidgets.QLineEdit(self.centralwidget)
        self.lineEditServerHostName.setObjectName("lineEditServerHostName")
        self.horizontalLayout_2.addWidget(self.lineEditServerHostName)
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.lineEditServerPort = QtWidgets.QLineEdit(self.centralwidget)
        self.lineEditServerPort.setMinimumSize(QtCore.QSize(61, 0))
        self.lineEditServerPort.setObjectName("lineEditServerPort")
        self.horizontalLayout_2.addWidget(self.lineEditServerPort)
        self.pushButtonConnectToServer = QtWidgets.QPushButton(self.centralwidget)
        self.pushButtonConnectToServer.setObjectName("pushButtonConnectToServer")
        self.horizontalLayout_2.addWidget(self.pushButtonConnectToServer)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.imageShowWidget = ImageShowBasicWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.imageShowWidget.sizePolicy().hasHeightForWidth())
        self.imageShowWidget.setSizePolicy(sizePolicy)
        self.imageShowWidget.setObjectName("imageShowWidget")
        self.verticalLayout.addWidget(self.imageShowWidget)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.labelFrameCoordsUnderMouse = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameCoordsUnderMouse.setObjectName("labelFrameCoordsUnderMouse")
        self.gridLayout_2.addWidget(self.labelFrameCoordsUnderMouse, 0, 0, 1, 1)
        self.labelFrameValueUnderMouse = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameValueUnderMouse.setObjectName("labelFrameValueUnderMouse")
        self.gridLayout_2.addWidget(self.labelFrameValueUnderMouse, 0, 1, 1, 1)
        self.labelRefreshRate = QtWidgets.QLabel(self.centralwidget)
        self.labelRefreshRate.setObjectName("labelRefreshRate")
        self.gridLayout_2.addWidget(self.labelRefreshRate, 0, 2, 1, 1)
        self.labelFrameValue5percentile = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameValue5percentile.setObjectName("labelFrameValue5percentile")
        self.gridLayout_2.addWidget(self.labelFrameValue5percentile, 1, 0, 1, 1)
        self.labelFrameValue95percentile = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameValue95percentile.setObjectName("labelFrameValue95percentile")
        self.gridLayout_2.addWidget(self.labelFrameValue95percentile, 1, 1, 1, 1)
        self.labelFrameCounter = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameCounter.setObjectName("labelFrameCounter")
        self.gridLayout_2.addWidget(self.labelFrameCounter, 1, 2, 1, 1)
        self.labelFrameValueMin = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameValueMin.setObjectName("labelFrameValueMin")
        self.gridLayout_2.addWidget(self.labelFrameValueMin, 2, 0, 1, 1)
        self.labelFrameValueMax = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameValueMax.setObjectName("labelFrameValueMax")
        self.gridLayout_2.addWidget(self.labelFrameValueMax, 2, 1, 1, 1)
        self.labelFrameShape = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameShape.setObjectName("labelFrameShape")
        self.gridLayout_2.addWidget(self.labelFrameShape, 2, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout_2)
        self.groupBox = QtWidgets.QGroupBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setMinimumSize(QtCore.QSize(481, 134))
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.comboBoxRequestedBinning = QtWidgets.QComboBox(self.groupBox)
        self.comboBoxRequestedBinning.setObjectName("comboBoxRequestedBinning")
        self.gridLayout.addWidget(self.comboBoxRequestedBinning, 2, 3, 1, 1)
        self.lineEditCurrentBinning = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditCurrentBinning.setObjectName("lineEditCurrentBinning")
        self.gridLayout.addWidget(self.lineEditCurrentBinning, 2, 1, 1, 2)
        self.labelExposureTimeInMs = QtWidgets.QLabel(self.groupBox)
        self.labelExposureTimeInMs.setObjectName("labelExposureTimeInMs")
        self.gridLayout.addWidget(self.labelExposureTimeInMs, 0, 0, 1, 2)
        self.labelFramerateInHz = QtWidgets.QLabel(self.groupBox)
        self.labelFramerateInHz.setObjectName("labelFramerateInHz")
        self.gridLayout.addWidget(self.labelFramerateInHz, 1, 0, 1, 1)
        self.lineEditCurrentExposureTime = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditCurrentExposureTime.setObjectName("lineEditCurrentExposureTime")
        self.gridLayout.addWidget(self.lineEditCurrentExposureTime, 0, 2, 1, 1)
        self.lineEditCurrentFramerate = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditCurrentFramerate.setObjectName("lineEditCurrentFramerate")
        self.gridLayout.addWidget(self.lineEditCurrentFramerate, 1, 1, 1, 2)
        self.labelBinning = QtWidgets.QLabel(self.groupBox)
        self.labelBinning.setObjectName("labelBinning")
        self.gridLayout.addWidget(self.labelBinning, 2, 0, 1, 1)
        self.lineEditRequestedExposureTime = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditRequestedExposureTime.sizePolicy().hasHeightForWidth())
        self.lineEditRequestedExposureTime.setSizePolicy(sizePolicy)
        self.lineEditRequestedExposureTime.setObjectName("lineEditRequestedExposureTime")
        self.gridLayout.addWidget(self.lineEditRequestedExposureTime, 0, 3, 1, 1)
        self.lineEditRequestedFramerate = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditRequestedFramerate.setObjectName("lineEditRequestedFramerate")
        self.gridLayout.addWidget(self.lineEditRequestedFramerate, 1, 3, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 1, 4, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout)
        self.verticalLayout.addWidget(self.groupBox)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar()
        self.menubar.setGeometry(QtCore.QRect(0, 0, 505, 22))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtCompat.translate("MainWindow", "MainWindow", None, -1))
        self.label_3.setText(QtCompat.translate("MainWindow", "Server", None, -1))
        self.label.setText(QtCompat.translate("MainWindow", "Hostname", None, -1))
        self.lineEditServerHostName.setText(QtCompat.translate("MainWindow", "192.168.1.1", None, -1))
        self.label_2.setText(QtCompat.translate("MainWindow", "Port", None, -1))
        self.lineEditServerPort.setText(QtCompat.translate("MainWindow", "7100", None, -1))
        self.pushButtonConnectToServer.setText(QtCompat.translate("MainWindow", "Connect", None, -1))
        self.labelFrameCoordsUnderMouse.setText(QtCompat.translate("MainWindow", "x,y: 1000,1000", None, -1))
        self.labelFrameValueUnderMouse.setText(QtCompat.translate("MainWindow", "value: 99999", None, -1))
        self.labelRefreshRate.setText(QtCompat.translate("MainWindow", "Refresh rate: 1000 Hz", None, -1))
        self.labelFrameValue5percentile.setText(QtCompat.translate("MainWindow", "5%: 100000", None, -1))
        self.labelFrameValue95percentile.setText(QtCompat.translate("MainWindow", "95%: 100000", None, -1))
        self.labelFrameCounter.setText(QtCompat.translate("MainWindow", "Frame counter: 100000000", None, -1))
        self.labelFrameValueMin.setText(QtCompat.translate("MainWindow", "Min: 100000", None, -1))
        self.labelFrameValueMax.setText(QtCompat.translate("MainWindow", "Max: 100000", None, -1))
        self.labelFrameShape.setText(QtCompat.translate("MainWindow", "Frame size: 1000 x 1000", None, -1))
        self.groupBox.setTitle(QtCompat.translate("MainWindow", "Controls", None, -1))
        self.labelExposureTimeInMs.setText(QtCompat.translate("MainWindow", "Exposure Time [ms]", None, -1))
        self.labelFramerateInHz.setText(QtCompat.translate("MainWindow", "Framerate [Hz]", None, -1))
        self.labelBinning.setText(QtCompat.translate("MainWindow", "Binning", None, -1))


from pysilico.gui.image_show_widget.image_show_basic_widget import ImageShowBasicWidget



#!/usr/bin/env python

from pysilico.client.camera_client import CameraClient
from pysilico.utils.timeout import Timeout


class Ocam2kClient(CameraClient):

    def __init__(self, rpcHandler, sockets):
        super().__init__(rpcHandler, sockets)

    def get_emgain(self, timeoutInSec=Timeout.GENERIC_COMMAND):
        return self._rpcHandler.sendRequest(
            self.request_socket,
            'get_emgain', [],
            timeout=timeoutInSec)




from plico.client.discovery import plico_list, plico_get, plico_client
from pysilico.client.client_map import client_map
from pysilico.client.camera_client import CameraClient
from pysilico.utils.constants import Constants


def _getDefaultConfigFilePath():
    from plico.utils.config_file_manager import ConfigFileManager
    cfgFileMgr= ConfigFileManager(Constants.APP_NAME,
                                  Constants.APP_AUTHOR,
                                  Constants.THIS_PACKAGE)
    return cfgFileMgr.getConfigFilePath()


defaultConfigFilePath= _getDefaultConfigFilePath()


def camera(hostname, port):
    '''Generic CameraClient, kept for backward compatibility'''
    return plico_client(CameraClient, hostname, port)


def list_cameras(timeout_in_seconds=2):
    '''List all available pysilico servers'''
    return plico_list(server_type='pysilico', timeout_in_seconds=timeout_in_seconds)


def get(dm_name, timeout_in_seconds=2):
    '''Get a client for a specific pysilico server'''
    return plico_get(server_type='pysilico', name=dm_name, default_class=CameraClient,
               timeout_in_seconds=timeout_in_seconds, client_map=client_map)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'image_show_basic_widget.ui',
# licensing of 'image_show_basic_widget.ui' applies.
#
# Created: Wed Sep 12 00:04:59 2018
#      by: pyside2-uic  running on PySide2 5.11.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtWidgets import QApplication as QtCompat

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(502, 359)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout.setObjectName("verticalLayout")
        self.graphicsView = GraphicsView(Form)
        self.graphicsView.setEnabled(True)
        self.graphicsView.setObjectName("graphicsView")
        self.verticalLayout.addWidget(self.graphicsView)
        self.labelInfo = QtWidgets.QLabel(Form)
        self.labelInfo.setText("")
        self.labelInfo.setObjectName("labelInfo")
        self.verticalLayout.addWidget(self.labelInfo)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtCompat.translate("Form", "Form", None, -1))

from pyqtgraph.widgets.GraphicsView import GraphicsView



#!/usr/bin/env python
import threading
import time
from pysilico.client.abstract_camera_client import AbstractCameraClient,\
    MultipleFrameRetriever, SnapshotEntry
from plico.rpc.abstract_remote_procedure_call import \
    AbstractRemoteProcedureCall
from plico.utils.logger import Logger
from plico.utils.decorator import override, returns, returnsForExample
from pysilico.types.camera_status import CameraStatus
from pysilico.utils.timeout import Timeout
from pysilico.types.camera_frame import CameraFrame
from plico.utils.snapshotable import Snapshotable
from plico.client.serverinfo_client import ServerInfoClient
from plico.client.hackerable_client import HackerableClient
import astropy.units as u



class CameraClient(AbstractCameraClient,
                   HackerableClient,
                   ServerInfoClient):

    def __init__(self,
                 rpcHandler,
                 sockets):
        assert isinstance(rpcHandler, AbstractRemoteProcedureCall)

        self._rpcHandler= rpcHandler
        self._subscriberSocket= sockets.serverSubscriber()
        self._subscriberSocketAddress= sockets.serverSubscriberAddress()
        self._subscriberSocket.disconnect(self._subscriberSocketAddress)
        self._displaySocket= sockets.serverDisplay()
        self._displaySocketAddress= sockets.serverDisplayAddress()
        self._requestSocket= sockets.serverRequest()
        self._statusSocket= sockets.serverStatus()
        self._logger= Logger.of('Camera client')
        HackerableClient.__init__(self,
                                  self._rpcHandler,
                                  self._requestSocket,
                                  self._logger)
        ServerInfoClient.__init__(self,
                                  self._rpcHandler,
                                  self._requestSocket,
                                  self._logger)
        self._shape= None
        self._dtype= None
        self._lastFrame= None
        self._timer = threading.Timer(1.0, self._build_methods)
        self._timer.start()

    def _build_methods(self):
        while True:
            try:
                status = self.getStatus()
                break
            except Exception:
                time.sleep(1)
        par_names = status.parameters.keys()
        for par_name in par_names:
            try:
                set_func = lambda self, value, name=par_name: self.setParameter(name, value)
                get_func = lambda self, name=par_name: self.getStatus().parameters[name]
                set_name = 'set' + par_name[0].upper() + par_name[1:]
                get_name = 'get' + par_name[0].upper() + par_name[1:]
                setattr(self, set_name, set_func.__get__(self, self.__class__))
                setattr(self, get_name, get_func.__get__(self, self.__class__))
            except Exception as e:
                print(f'Error creating method for parameter {par_name}:', e)

    @override
    @returns(CameraStatus)
    def getStatus(
            self, timeoutInSec=Timeout.GET_FRAME_FOR_DISPLAY_TIMEOUT):
        '''
        Returns the current status of the camera.
        This includes frame dimensions, exposure time, binning, and other parameters.'''
        return self._rpcHandler.receivePickable(
            self._statusSocket,
            timeoutInSec)


    @override
    @returns(CameraFrame)
    def getFrameForDisplay(
            self, timeoutInSec=Timeout.GET_FRAME_FOR_DISPLAY_TIMEOUT):
        '''
        Returns a single frame from the camera for display purposes.
        This method is typically used to retrieve the most recent frame for visualization.'''
        return self._rpcHandler.recvCameraFrame(
            self._displaySocket,
            timeoutInSec=timeoutInSec)


    def _getLastAvailableFrame(self):
        self._lastFrame= self._rpcHandler.recvCameraFrame(
            self._subscriberSocket,
            timeoutInSec=Timeout.GET_FRAME_TIMEOUT)
        return self._lastFrame



    @override
    @returns(CameraFrame)
    def getFutureFrames(self,
                        numberOfReturnedImages,
                        numberOfFramesToAverageForEachImage=1,
                        timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        '''
        Retrieves a specified number of frames from the camera, averaging multiple frames for each image.
        Args:
            numberOfReturnedImages (int): The number of images to return.
            numberOfFramesToAverageForEachImage (int): The number of frames to average for each image.
            timeoutSec (float): Timeout for the operation in seconds.
        Returns:
            list: A list of CameraFrame objects, each representing an averaged image.'''
        self._subscriberSocket.connect(self._subscriberSocketAddress)
        trashFirst= self._getLastAvailableFrame()
        frames= MultipleFrameRetriever.getFrames(
            numberOfReturnedImages,
            numberOfFramesToAverageForEachImage,
            self._getLastAvailableFrame)
        self._subscriberSocket.disconnect(self._subscriberSocketAddress)
        return frames


    @override
    @returnsForExample((1024, 1024))
    def shape(self):
        '''
        Returns the shape of the camera frames as a tuple (height, width).
        This method retrieves the current frame dimensions from the camera status.'''
        status= self.getStatus()
        return (status.frameHeight, status.frameWidth)


    def dtype(self):
        '''
        Returns the data type of the camera frames.
        This method retrieves the data type from the camera status.'''
        return self.getStatus().dtype


    @property 
    def subscriber_socket(self):
        '''
        Returns the subscriber socket for the camera client.
        This socket is used to receive frames from the camera server.'''
        return self._subscriberSocket

    @property 
    def request_socket(self):
        '''
        Returns the request socket for the camera client.
        This socket is used to send requests to the camera server.'''
        return self._requestSocket

    @override
    def setExposureTime(self,
                        exposureTimeInMilliSeconds,
                        timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        '''
        Sets the exposure time for the camera.
        Args:
            exposureTimeInMilliSeconds (float): The exposure time in milliseconds.
            timeoutSec (float): Timeout for the operation in seconds.
        '''
        self._logger.notice("Setting exposure time to %f ms" %
                            exposureTimeInMilliSeconds)
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'setExposureTime',
            [exposureTimeInMilliSeconds],
            timeout=timeoutSec)


    @override
    @returns(float)
    def exposureTime(self, unit=False,
                     timeoutInSec=Timeout.GET_FRAME_FOR_DISPLAY_TIMEOUT):
        '''
        Returns the current exposure time of the camera.
        Args:
            unit (bool): If True, returns the exposure time in astropy units.
            timeoutInSec (float): Timeout for the operation in seconds.
        Returns:
            float or astropy.units.Quantity: The exposure time in milliseconds, either as a float or
            an astropy Quantity depending on the unit parameter.
        '''
        if unit == False:
            return float(self.getStatus(timeoutInSec=timeoutInSec).exposureTimeInMilliSec)
        else:
            return float(self.getStatus(timeoutInSec=timeoutInSec).exposureTimeInMilliSec) * u.ms


    @override
    def getSnapshot(self,
                    prefix,
                    timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        '''
        Retrieves a snapshot of the camera status.
        Args:
            prefix (str): A prefix to prepend to the snapshot entries.
            timeoutSec (float): Timeout for the operation in seconds.
        Returns:
            dict: A dictionary containing the camera status snapshot with the specified prefix.
        '''
        status= self.getStatus(timeoutInSec=timeoutSec)
        self._logger.notice("Getting snapshot for %s " % prefix)
        return self._createSnapshotFromStatus(prefix, status)


    def _createSnapshotFromStatus(self, prefix, status):
        assert isinstance(status, CameraStatus)
        snapshot= {}
        snapshot[SnapshotEntry.CAMERA_NAME]= status.name
        snapshot[SnapshotEntry.EXPOSURE_TIME_MS]= status.exposureTimeInMilliSec
        snapshot[SnapshotEntry.FRAME_WIDTH_PX]= status.frameWidth
        snapshot[SnapshotEntry.FRAME_HEIGHT_PX]= status.frameHeight
        snapshot[SnapshotEntry.BINNING]= status.binning
        snapshot[SnapshotEntry.PARAMETERS]= status.parameters
        return Snapshotable.prepend(prefix, snapshot)



    @override
    def setBinning(self,
                   binning,
                   timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        '''
        Sets the binning factor for the camera.
        Args:
            binning (int): The binning factor to set.
            timeoutSec (float): Timeout for the operation in seconds.
        '''
        self._logger.notice("Setting binning to %d " % binning)
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'setBinning',
            [binning],
            timeout=timeoutSec)


    @override
    @returns(int)
    def getBinning(self, timeoutInSec=Timeout.GET_FRAME_FOR_DISPLAY_TIMEOUT):
        '''
        Returns the current binning factor of the camera.
        Args:
            timeoutInSec (float): Timeout for the operation in seconds.
        Returns:
            int: The current binning factor.
        '''
        return int(self.getStatus(timeoutInSec=timeoutInSec).binning)


    @override
    def setFrameRate(self, frameRate,
                     timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        '''
        Sets the frame rate for the camera.
        Args:
            frameRate (float): The desired frame rate in frames per second.
            timeoutSec (float): Timeout for the operation in seconds.
        '''
        self._logger.notice("Setting frameRate to %f" % frameRate)
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'setFrameRate',
            [frameRate],
            timeout=timeoutSec)


    @override
    @returns(float)
    def getFrameRate(self, timeoutInSec=Timeout.GET_FRAME_FOR_DISPLAY_TIMEOUT):
        '''
        Returns the current frame rate of the camera.
        Args:
            timeoutInSec (float): Timeout for the operation in seconds.
        Returns:
            float: The current frame rate in frames per second.'''
        return float(self.getStatus(timeoutInSec=timeoutInSec).frameRate)


    @override
    @returns(CameraFrame)
    def getDarkFrame(self, timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        '''
        Retrieves the current dark frame from the camera.
        Returns:
            CameraFrame: The current dark frame.
        '''
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'getDarkFrame',
            [],
            timeout=timeoutSec)


    @override
    def setDarkFrame(self, darkFrame,
                     timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        '''
        Sets the dark frame for the camera.
        Args:
            darkFrame (CameraFrame): The dark frame to set.
            timeoutSec (float): Timeout for the operation in seconds.
        '''
        assert isinstance(darkFrame, CameraFrame)
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'setDarkFrame',
            [darkFrame],
            timeout=timeoutSec)

    @override
    def setParameter(self, name, value,
                     timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        '''
        Sets a parameter for the camera.
        Args:
            name (str): The name of the parameter to set.
            value: The value to set for the parameter.
            timeoutSec (float): Timeout for the operation in seconds.
        '''
        assert isinstance(name, str)
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'setParameter',
            [name, value],
            timeout=timeoutSec)


    def terminate(self):
        '''
        Terminates the camera client.
        This method is called to clean up resources and close connections.'''
        self._logger.notice("Terminating camera client")
        if hasattr(self, '_timer'):
            self._timer.cancel()
        self._subscriberSocket.close(linger=0)
        self._displaySocket.close(linger=0)
        self._requestSocket.close(linger=0)
        self._statusSocket.close(linger=0)
        self._logger.notice("Camera client terminated")


import sys
import numpy as np
from pysilico.gui.image_show_widget.image_show_basic_widget import ColorMaps

import pysilico
from plico.rpc.zmq_remote_procedure_call import ZmqRpcTimeoutError
from pysilico.types.camera_frame import CameraFrame


def _importPyQt5():
    from PyQt5.Qt import QApplication
    from PyQt5 import QtCore
    from PyQt5 import QtWidgets
    from PyQt5 import uic

    def loadUiWidget(uifilename, parent=None):
        return uic.loadUi(uifilename, parent)

    return QApplication, QtCore, QtWidgets, loadUiWidget, QtCore.pyqtSlot


def _importPySide2():
    from PySide2.QtWidgets import QApplication
    from PySide2 import QtCore
    from PySide2 import QtWidgets
    from PySide2 import QtUiTools

    def loadUiWidget(uifilename, parent=None):
        loader = QtUiTools.QUiLoader()
        uifile = QtCore.QFile(uifilename)
        uifile.open(QtCore.QFile.ReadOnly)
        ui = loader.load(uifile, parent)
        uifile.close()
        return ui

    return QApplication, QtCore, QtWidgets, loadUiWidget, QtCore.Slot


def _importQtPy():
    import os
    
    # 211123 workaround for window not showing bug. When can this be removed? Bah...
    # https://stackoverflow.com/questions/64818879/is-there-any-solution-regarding-to-pyqt-library-doesnt-work-in-mac-os-big-sur
    os.environ["QT_MAC_WANTS_LAYER"]='1'

    if "QT_PREFERRED_BINDING" not in os.environ:
        os.environ["QT_PREFERRED_BINDING"] = os.pathsep.join(
            ["PySide2", "PyQt5", "PySide", "PyQt4"]
            #["PyQt5", "PySide2", "PySide", "PyQt4"]
        )

    import Qt
    from Qt.QtWidgets import QApplication
    from Qt import QtCore
    from Qt import QtWidgets
    from Qt import QtCompat
    from Qt.QtCore import Slot

    def loadUiWidget(uifilename, parent=None):
        ui = QtCompat.loadUi(uifile=uifilename, baseinstance=parent)
        return ui

    print("Qt binding %s" % Qt.__binding__)
    print("Qt version %s" % Qt.__qt_version__)
    print("Qt binding version %s" % Qt.__binding_version__)

    return QApplication, QtCore, QtWidgets, loadUiWidget, Slot


#QApplication, QtCore, QtWidgets, loadUiWidget, Slot= _importPySide2()
#QApplication, QtCore, QtWidgets, loadUiWidget, Slot= _importQtPy()
QApplication, QtCore, QtWidgets, loadUiWidget, Slot= _importPyQt5()




class CameraControlGui(QtWidgets.QMainWindow):

    def __init__(self):
        super(CameraControlGui, self).__init__()
        self._camera= pysilico.camera('192.168.1.1', 7100)

        self._lastFrame= CameraFrame(np.zeros((10, 10)), 42)

        self._intervalUpper= 1
        self._intervalRefresh= 500
        self._intervalRefreshCameraStatus= 1000

        self._linesPlot1D= None
        self._imShow2D= None

        self._counterUpper= 0
        self._lastCounterUpper= 0

        from pysilico.gui.camera_control_gui_ui import Ui_MainWindow
        self._ui = Ui_MainWindow()
        self._ui.setupUi(self)

        print(self)
        print(self._ui)
        print(self._ui.imageShowWidget)


        self._setUpConnectButton()
        self._setUpExposureTimeLineEdit()
        self._setUpFrameRateLineEdit()
        self._setUpRequestedBinningComboBox()
        self._setUpSpeedMeter()
        self._setUpCameraStatus()
        self._setUpPlotUpper()


    def keyPressEvent(self, event):
        key = event.key()
        print("event key %s" % str(key))
        if key == QtCore.Qt.Key_Escape:
            self.close()
        if key == QtCore.Qt.Key_S:
            self._timerUpper.stop()
        if key == QtCore.Qt.Key_G:
            self._timerUpper.start()
        super(CameraControlGui, self).keyPressEvent(event)



    def _setUpConnectButton(self):
        self._ui.pushButtonConnectToServer.clicked.connect(
            self._onpushButtonConnectToServerClick)


    def _setUpExposureTimeLineEdit(self):
        self._ui.lineEditRequestedExposureTime.editingFinished.connect(
            self._onlineEditRequestedExposureTimeEditingFinished)


    def _setUpFrameRateLineEdit(self):
        self._ui.lineEditRequestedFramerate.editingFinished.connect(
            self._onlineEditRequestedFrameRateEditingFinished)


    def _setUpRequestedBinningComboBox(self):
        self._ui.comboBoxRequestedBinning.addItems(
            ["1", "2", "3", "4", "8", "16", "32"])
        self._ui.comboBoxRequestedBinning.currentIndexChanged.connect(
            self._oncomboBoxRequestedBinningCurrentIndexChanged)


    @Slot()
    def _onpushButtonConnectToServerClick(self):
        self._camera= pysilico.camera(
            self._ui.lineEditServerHostName.text(),
            int(self._ui.lineEditServerPort.text()))


    @Slot()
    def _onlineEditRequestedExposureTimeEditingFinished(self):
        self._camera.setExposureTime(
            float(self._ui.lineEditRequestedExposureTime.text()))


    @Slot()
    def _onlineEditRequestedFrameRateEditingFinished(self):
        self._camera.setFrameRate(
            float(self._ui.lineEditRequestedFramerate.text()))


    @Slot()
    def _oncomboBoxRequestedBinningCurrentIndexChanged(self):
        self._camera.setBinning(
            int(self._ui.comboBoxRequestedBinning.currentText()))


    def _setUpSpeedMeter(self):
        self._timerSpeedMeter= QtCore.QTimer()
        self._timerSpeedMeter.timeout.connect(self._executeRefreshSpeedMeter)
        self._timerSpeedMeter.start(self._intervalRefresh)


    def _setUpCameraStatus(self):
        self._ui.lineEditCurrentExposureTime.setReadOnly(True)
        self._ui.lineEditCurrentFramerate.setReadOnly(True)
        self._ui.lineEditCurrentBinning.setReadOnly(True)
        self._timerCameraStatus= QtCore.QTimer()
        self._timerCameraStatus.timeout.connect(
            self._executeRefreshCameraStatus)
        self._timerCameraStatus.start(self._intervalRefreshCameraStatus)



    def _refreshSpeed(self, label, counterNow,
                      counterBefore, intervalInMilliSec):
        rate= (counterNow - counterBefore) / intervalInMilliSec * 1e3
        label.setText("Refresh rate: %g Hz" % rate)


    def _executeRefreshSpeedMeter(self):
        self._refreshSpeed(self._ui.labelRefreshRate,
                           self._counterUpper,
                           self._lastCounterUpper,
                           self._intervalRefresh)
        self._lastCounterUpper= self._counterUpper


    def _executeRefreshCameraStatus(self):
        try:
            status= self._camera.getStatus()
            expTime= float(status.exposureTimeInMilliSec)
            frameRate= float(status.frameRate)
            binning= status.binning
            self._ui.lineEditCurrentExposureTime.setText("%.3f" % expTime)
            self._ui.lineEditCurrentFramerate.setText("%.3f" % frameRate)
            self._ui.lineEditCurrentBinning.setText("%.3f" % binning)
            self._ui.labelFrameShape.setText(
                "Frame shape %d x %d" % (status.frameWidth,
                                         status.frameHeight))
        except ZmqRpcTimeoutError:
            pass


    def _tryToGetFrameFromCamera(self):
        try:
            self._lastFrame= self._camera.getFrameForDisplay()
        except ZmqRpcTimeoutError:
            pass


    def _executePlotUpper(self):
        self._tryToGetFrameFromCamera()
        frame= self._lastFrame.toNumpyArray().T
        self._ui.imageShowWidget.setImage(frame)
        self._ui.labelFrameCounter.setText(
            "Frame counter: %d" % self._lastFrame.counter())

        percent= np.percentile(frame, [0, 5, 50, 95, 100])
        self._ui.labelFrameValueMax.setText("Max: %g" % percent[4])
        self._ui.labelFrameValueMin.setText("Min: %g" % percent[0])
        self._ui.labelFrameValue5percentile.setText(
            "5%%: %g" % percent[1])
        self._ui.labelFrameValue95percentile.setText(
            "95%%: %g" % percent[3])
        self._counterUpper+= 1


    def _setUpPlotUpper(self):
        self._ui.imageShowWidget.setBackgroundColor('w')
        self._ui.imageShowWidget.setColormap(ColorMaps.GREY)
        self._ui.imageShowWidget.setImageSaturationValues(
            40, 4000)
        self._ui.imageShowWidget.enableSaturationOverlay(False)


        self._timerUpper= QtCore.QTimer()
        self._timerUpper.timeout.connect(self._executePlotUpper)
        self._timerUpper.start(self._intervalUpper)



class Runner(object):


    def __init__(self):
        pass


    def _setUp(self, argv):
        pass


    def run(self, argv):
        self._setUp(argv)
        qtApp = QApplication(argv)
        self._gui = CameraControlGui()
        self._gui.show()
        qtApp.exec_()


    def terminate(self, signal, frame):
        QApplication.quit()


if __name__ == '__main__':
    runner = Runner()
    sys.exit(runner.run(sys.argv[1:]))






#!/usr/bin/env python
import os
import unittest
import shutil

import numpy as np
import astropy.io.fits as pyfits
from pysilico.calibration.calibration_manager import CalibrationManager
from pysilico.types.camera_frame import CameraFrame


class CalibrationManagerTest(unittest.TestCase):

    CALIB_DIR = "./calib_tmp"

    def _removeCalibrationDir(self):
        if os.path.exists(self.CALIB_DIR):
            shutil.rmtree(self.CALIB_DIR)

    def setUp(self):
        self._removeCalibrationDir()
        self.calibMgr = CalibrationManager(self.CALIB_DIR)

    def tearDown(self):
        self._removeCalibrationDir()

    def test_storage_of_dark_frame(self):
        hdr = pyfits.Header()
        hdr.update({"zzz": 42})

        darkCameraFrameArray = np.arange(100).reshape((10, 10))
        darkCameraFrame = CameraFrame(darkCameraFrameArray)
        self.calibMgr.saveDarkFrame('tagDarkFrame', darkCameraFrame, hdr)
        self.assertTrue(os.path.exists(
            os.path.join(self.CALIB_DIR, "dark_frame",
                         "tagDarkFrame.fits")))

        loaded = self.calibMgr.loadDarkFrame("tagDarkFrame")
        self.assertTrue(np.array_equal(
            darkCameraFrame.toNumpyArray(),
            loaded.toNumpyArray()))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import unittest
import numpy as np
from pysilico.calibration.in_memory_calibration_manager \
    import InMemoryCalibrationManager
from pysilico.types.camera_frame import CameraFrame


__version__= "$Id: in_memory_calibration_manager_test.py 233 2017-01-22 23:38:47Z lbusoni $"


class InMemoryCalibrationManagerTest(unittest.TestCase):

    def setUp(self):
        self.calibMgr= InMemoryCalibrationManager()


    def test_dark_frame_storage(self):
        darkCameraFrameArray=np.arange(100).reshape((10, 10))
        darkCameraFrame= CameraFrame(darkCameraFrameArray)
        self.calibMgr.saveDarkFrame('tagDarkFrame', darkCameraFrame)

        loaded= self.calibMgr.loadDarkFrame("tagDarkFrame")
        self.assertTrue(np.array_equal(
            darkCameraFrame.toNumpyArray(),
            loaded.toNumpyArray()))


if __name__ == "__main__":
    unittest.main()



from pysilico.client.abstract_camera_client import AbstractCameraClient,\
    MultipleFrameRetriever, SnapshotEntry
from plico.utils.decorator import override, returns
from pysilico.types.camera_frame import CameraFrame
from plico.utils.snapshotable import Snapshotable
from pysilico.types.camera_status import CameraStatus
from pysilico_server.devices.simulated_auxiliary_camera import \
    SimulatedAuxiliaryCamera



class SimulatedCameraClient(AbstractCameraClient):

    def __init__(self):
        self._name= 'mySimulatedCamera'
        self._cameraDevice= SimulatedAuxiliaryCamera(self._name)
        self._frameToReturn= None
        self._darkFrame= None


    def deinitialize(self):
        self._cameraDevice.stopAcquisition()
        self._cameraDevice.deinitialize()


    def frameCounter(self):
        return self._cameraDevice.frameCounter()


    def simulateCameraFrame(self, cameraFrameToReturn):
        self._frameToReturn= cameraFrameToReturn


    @override
    @returns(CameraFrame)
    def getFrameForDisplay(self):
        return CameraFrame(self._cameraDevice.readFrame())


    def _getCorrectedFrame(self):
        return CameraFrame(self._cameraDevice.readFrame())


    @override
    @returns(CameraFrame)
    def getFutureFrames(self,
                        numberOfReturnedImages,
                        numberOfFramesToAverageForEachImage=1):
        if self._frameToReturn is not None:
            return self._frameToReturn
        else:
            return MultipleFrameRetriever.getFrames(
                numberOfReturnedImages,
                numberOfFramesToAverageForEachImage,
                self._getCorrectedFrame)


    @override
    def setExposureTime(self, exposureTimeInMilliSeconds):
        self._cameraDevice.setExposureTime(exposureTimeInMilliSeconds)


    @override
    def exposureTime(self):
        return self._cameraDevice.exposureTime()


    @override
    def shape(self):
        return (self._cameraDevice.rows(), self._cameraDevice.cols())


    @override
    def getSnapshot(self, prefix):
        snapshot= {}
        snapshot[SnapshotEntry.EXPOSURE_TIME_MS]= self.exposureTime()
        snapshot[SnapshotEntry.FRAME_WIDTH_PX]= self._cameraDevice.cols()
        snapshot[SnapshotEntry.FRAME_HEIGHT_PX]= self._cameraDevice.rows()
        return Snapshotable.prepend(prefix, snapshot)



    @override
    def setBinning(self, binning):
        self._cameraDevice.setBinning(binning)


    @override
    def getBinning(self):
        return self._cameraDevice.getBinning()


    def simulatePupilCenterInUnbinnedPixels(self, center):
        self._cameraDevice.setPupilsCenterInUnbinnedPixels(center)


    def simulatePupilsRadiusInUnbinnedPixels(self, radius):
        self._cameraDevice.setPupilsRadiusInUnbinnedPixels(radius)


    def simulateNoiseInCount(self, noise):
        self._cameraDevice.setNoiseInCount(noise)


    def simulateTilt(self, z2and3Coefficients):
        self._cameraDevice.setTilt(z2and3Coefficients)


    @override
    @returns(CameraStatus)
    def getStatus(self):
        status= CameraStatus(self._name,
                             self._cameraDevice.cols(),
                             self._cameraDevice.rows(),
                             self._cameraDevice.dtype(),
                             self.getBinning(),
                             self.exposureTime())
        return status

    @override
    def setParameter(self, name, value):
        '''We don't have any'''
        raise Exception('Parameter %s is not valid' % str(name))


    @override
    def setFrameRate(self, frameRate):
        self._cameraDevice.setFrameRate(frameRate)


    @override
    def getFrameRate(self):
        return self._cameraDevice.getFrameRate()

    @override
    @returns(CameraFrame)
    def getDarkFrame(self):
        return self._darkFrame

    @override
    def setDarkFrame(self, darkFrame):
        self._darkFrame= darkFrame



import abc
from six import with_metaclass
import numpy as np
from plico.utils.decorator import returnsNone, returns, returnsForExample
from pysilico.types.camera_frame import CameraFrame
from pysilico.types.camera_status import CameraStatus


class SnapshotEntry(object):
    EXPOSURE_TIME_MS= "EXPOSURE_TIME_MS"
    FRAME_WIDTH_PX= "WIDTH_PX"
    FRAME_HEIGHT_PX= "HEIGHT_PX"
    CAMERA_NAME= 'NAME'
    BINNING= "BINNING"
    PARAMETERS="PARAMETERS"


class MultipleFrameRetriever():

    @classmethod
    def _getAveragedFutureFrames(cls, nFramesToAverage, frameRetriever):
        a= []
        for _ in range(nFramesToAverage):
            frame= frameRetriever()
            a.append(frame.toNumpyArray())

        arr=np.dstack([x for x in a])
        lastCounter= frame.counter()
        return np.mean(arr, axis=2), lastCounter


    @classmethod
    def _getCube(cls, nFrames, nFramesToAverage, frameRetriever):
        a= []
        for _ in range(nFrames):
            frame, lastCounter= cls._getAveragedFutureFrames(nFramesToAverage,
                                                             frameRetriever)
            a.append(frame)

        arr=np.dstack([x for x in a])
        return arr, lastCounter

    @classmethod
    def getFrames(cls,
                  numberOfReturnedImages,
                  numberOfFramesToAverageForEachImage,
                  frameRetriever):
        cube, lastCounter= cls._getCube(
            numberOfReturnedImages,
            numberOfFramesToAverageForEachImage,
            frameRetriever)
        frame = CameraFrame.fromNumpyArray(cube.squeeze())
        frame.setCounter(lastCounter)
        return frame


class AbstractCameraClient(with_metaclass(abc.ABCMeta, object)):

    @abc.abstractmethod
    @returns(CameraFrame)
    def getFrameForDisplay(self):
        assert False


    @abc.abstractmethod
    @returns(CameraFrame)
    def getFutureFrames(self,
                        numberOfReturnedImages,
                        numberOfFramesToAverageForEachImage):
        assert False


    @abc.abstractmethod
    @returnsNone
    def setExposureTime(self, exposureTimeInMilliSeconds):
        assert False


    @abc.abstractmethod
    @returns(float)
    def exposureTime(self):
        assert False


    @abc.abstractmethod
    @returnsForExample((1024, 1024))
    def shape(self):
        assert False


    @abc.abstractmethod
    @returnsForExample({'WFS_CAMERA.EXPOSURE_TIME_MS': 10})
    def getSnapshot(self, prefix):
        assert False


    @abc.abstractmethod
    @returnsNone
    def setBinning(self, binning):
        assert False


    @abc.abstractmethod
    @returns(int)
    def getBinning(self):
        assert False


    @abc.abstractmethod
    @returns(CameraStatus)
    def getStatus(self):
        assert False


    @abc.abstractmethod
    @returnsNone
    def setFrameRate(self, frameRate):
        assert False


    @abc.abstractmethod
    @returns(float)
    def getFrameRate(self):
        assert False


    @abc.abstractmethod
    @returns(CameraFrame)
    def getDarkFrame(self):
        assert False


    @abc.abstractmethod
    def setDarkFrame(self, darkFrame):
        assert False

    @abc.abstractmethod
    def setParameter(self, name, value):
        '''Device-specific parameters'''
        assert False

